const express = require('express');
const fs      = require('fs');
const sys     = require('util');
const exec    = require('child_process').exec;

const app = express();

function runCmd(cmd) {
  exec(cmd, function(error, out, err) {
    if (!error) {
      console.log("Successfully executed -> " + cmd);
      console.log("Result: " + out);
    } else {
      console.log("Error: " + error);
    }
  });
};

/**
 *
 * @param {string} dir The application directory
 * @param {string} cmd The command to run
 *
 * @return {string}
 */
function fullCommand(dir, cmd) {
  return "cd " + dir + " && " + cmd;
}

// ------------------------------------

/**
 * Loads configuration from config.json
 *
 * @return {object}
 */
function loadConfig() {
  var data = fs.readFileSync('./config.json'),
      conf;
  try {
    conf = JSON.parse(data);
  } catch (err) {
    console.log('Error parsing config.json: ' + err);
  }
  return conf;
}

const config = loadConfig();

/**
 * Returns the full command to run
 *
 * @param {string} cmd the command to run
 * @return {string}
 */
function execCommand(cmd) {
   return "cd " + config["dir"] + " && " + cmd;
}

app.configure(function(){
  app.use('/media', express.static(__dirname + '/media'));
  app.use(express.static(__dirname + '/public'));
});

// ------------------------------------
// Routes
// ------------------------------------

app.get('/', function(req, res){
  res.sendfile(__dirname + '/public/index.html');
});

app.post('/test/all', function(req, res) {
  var c = execCommand(config["test"]["all"]);
  console.log("Running specs...");
  runCmd(c);
  res.send("Running specs...");
});

app.post('/db/migrate', function(req, res) {
  var c = execCommand(config["database"]["migrate"]);
  console.log("Running migrations...");
  runCmd(c);
  res.send("Running database migrations...");
});

app.post('/db/rollback', function(req, res) {
  var c = execCommand(config["database"]["rollback"]);
  console.log("Rolling back database...");
  runCmd(c);
  res.send("Rolling back database...");
});

app.post('/server/start', function(req, res) {
  var c = execCommand("rails s -p 3001");
  runCmd(c);
  res.send("Running server...");
});

// ------------------------------------
// APP STARTS HERE
// ------------------------------------

app.listen(config['nodePort']);

console.log("Server running on port " + config['nodePort']);
